# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawTransactionsClient
from ..types.transaction_type import TransactionType
from ..types.boolean_string import BooleanString
from ..types.transaction_state import TransactionState
from ..types.transaction_amount_details import TransactionAmountDetails
from ..types.transaction_origin_payment_details import TransactionOriginPaymentDetails
from ..types.transaction_destination_payment_details import TransactionDestinationPaymentDetails
from ..types.origin_funds_info import OriginFundsInfo
from ..types.device_data import DeviceData
from ..types.tag import Tag
from ..core.request_options import RequestOptions
from .types.transactions_verify_response import TransactionsVerifyResponse
from ..types.transaction_with_rules_result import TransactionWithRulesResult
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawTransactionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TransactionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTransactionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTransactionsClient
        """
        return self._raw_client

    def verify(
        self,
        *,
        type: TransactionType,
        transaction_id: str,
        timestamp: float,
        validate_origin_user_id: typing.Optional[BooleanString] = None,
        validate_destination_user_id: typing.Optional[BooleanString] = None,
        validate_transaction_id: typing.Optional[BooleanString] = None,
        trs_only: typing.Optional[BooleanString] = None,
        origin_user_id: typing.Optional[str] = OMIT,
        destination_user_id: typing.Optional[str] = OMIT,
        transaction_state: typing.Optional[TransactionState] = OMIT,
        origin_amount_details: typing.Optional[TransactionAmountDetails] = OMIT,
        destination_amount_details: typing.Optional[TransactionAmountDetails] = OMIT,
        origin_payment_details: typing.Optional[TransactionOriginPaymentDetails] = OMIT,
        destination_payment_details: typing.Optional[TransactionDestinationPaymentDetails] = OMIT,
        origin_funds_info: typing.Optional[OriginFundsInfo] = OMIT,
        related_transaction_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        product_type: typing.Optional[str] = OMIT,
        promotion_code_used: typing.Optional[bool] = OMIT,
        reference: typing.Optional[str] = OMIT,
        origin_device_data: typing.Optional[DeviceData] = OMIT,
        destination_device_data: typing.Optional[DeviceData] = OMIT,
        tags: typing.Optional[typing.Sequence[Tag]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TransactionsVerifyResponse:
        """
        ## POST Transactions

        `/transactions` endpoint allows you to operate on the [Transaction entity.](/guides/overview/entities#transaction)

        In order to pass the payload of a transaction to Flagright and verify the transaction, you will need to call this endpoint with the transaction payload. Not all fields are mandatory, you will only need to pass in the fields that you have and are relevant for your compliance setup.


        ### Payload

        Here are some of the most used payload fields explained (you can find the full payload [schema below](/api-reference/api-reference/transactions/verify#request) with 1 line descriptions):

        * `type`: Type of transaction (Ex: `WITHDRAWAL`, `DEPOSIT`, `TRANSFER` etc).
        * `transactionId` - Unique Identifier for the transaction.
        * `timestamp` - UNIX timestamp in *milliseconds* of when the transaction took place
        * `transactionState` - The state of the transaction, set to `CREATED` by default. [More details here](/guides/overview/entities#transaction-lifecycle-through-transaction-events)
        * `originUserId` - Unique identifier (if any) of the user who is sending the money. This user must be created within the Flagright system before using the [create a consumer user](/api-reference/api-reference/consumer-users/create) or [create a business user](/api-reference/api-reference/business-users/create) endpoint
        * `destinationUserId` - Unique identifier (if any) of the user who is receiving the money. This user must be created within the Flagright system before using the [create a consumer user](/api-reference/api-reference/consumer-users/create) or [create a business user](/api-reference/api-reference/business-users/create) endpoint
        * `originAmountDetails` - Details of the amount being sent from the origin
        * `destinationAmountDetails` - Details of the amount being received at the destination
        * `originPaymentDetails` - Payment details (if any) used at the origin (ex: `CARD`, `IBAN`, `WALLET` etc). You can click on the dropdown next to the field in the schema below to view all supported payment types.
        * `destinationPaymentDetails` - Payment details (if any) used at the destination (ex: `CARD`, `IBAN`, `WALLET` etc). You can click on the dropdown next to the field in the schema below to view all supported payment types.

        Parameters
        ----------
        type : TransactionType

        transaction_id : str
            Unique transaction identifier

        timestamp : float
            Timestamp of when transaction took place

        validate_origin_user_id : typing.Optional[BooleanString]
            Boolean string whether Flagright should validate if provided originUserId exist. True by default

        validate_destination_user_id : typing.Optional[BooleanString]
            Boolean string whether Flagright should validate if provided destinationUserId exist. True by default

        validate_transaction_id : typing.Optional[BooleanString]
            Boolean string whether Flagright should validate if provided transactionId exist. True by default

        trs_only : typing.Optional[BooleanString]

        origin_user_id : typing.Optional[str]
            UserId for where the transaction originates from

        destination_user_id : typing.Optional[str]
            UserId for transaction's destination. In other words, where the value is being transferred to.

        transaction_state : typing.Optional[TransactionState]

        origin_amount_details : typing.Optional[TransactionAmountDetails]

        destination_amount_details : typing.Optional[TransactionAmountDetails]

        origin_payment_details : typing.Optional[TransactionOriginPaymentDetails]
            Payment details of the origin. It can be a bank account number, wallet ID, card fingerprint etc.

        destination_payment_details : typing.Optional[TransactionDestinationPaymentDetails]

        origin_funds_info : typing.Optional[OriginFundsInfo]

        related_transaction_ids : typing.Optional[typing.Sequence[str]]
            IDs of transactions related to this transaction. Ex: refund, split bills

        product_type : typing.Optional[str]
            Type of produce being used by the consumer (ex wallets, payments etc)

        promotion_code_used : typing.Optional[bool]
            Whether a promotion code was used or not the transaction

        reference : typing.Optional[str]
            Reference field for the transaction indicating the purpose of the transaction etc.

        origin_device_data : typing.Optional[DeviceData]

        destination_device_data : typing.Optional[DeviceData]

        tags : typing.Optional[typing.Sequence[Tag]]
            Additional information that can be added via tags

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransactionsVerifyResponse
            Created

        Examples
        --------
        from flagright import (
            DeviceData,
            Flagright,
            Tag,
            TransactionAmountDetails,
            TransactionDestinationPaymentDetails_Card,
            TransactionOriginPaymentDetails_Card,
        )

        client = Flagright(
            api_key="YOUR_API_KEY",
        )
        client.transactions.verify(
            type="DEPOSIT",
            transaction_id="7b80a539eea6e78acbd6d458e5971482",
            timestamp=1641654664000.0,
            origin_user_id="8650a2611d0771cba03310f74bf6",
            destination_user_id="9350a2611e0771cba03310f74bf6",
            origin_amount_details=TransactionAmountDetails(
                transaction_amount=800.0,
                transaction_currency="EUR",
                country="DE",
            ),
            destination_amount_details=TransactionAmountDetails(
                transaction_amount=68351.34,
                transaction_currency="INR",
                country="IN",
            ),
            origin_payment_details=TransactionOriginPaymentDetails_Card(
                card_fingerprint="20ac00fed8ef913aefb17cfae1097cce",
                card_issued_country="TR",
                transaction_reference_field="Deposit",
                f_3_ds_done=True,
            ),
            destination_payment_details=TransactionDestinationPaymentDetails_Card(
                card_fingerprint="20ac00fed8ef913aefb17cfae1097cce",
                card_issued_country="TR",
                transaction_reference_field="Deposit",
                f_3_ds_done=True,
            ),
            promotion_code_used=True,
            reference="loan repayment",
            origin_device_data=DeviceData(
                battery_level=95.0,
                device_latitude=13.0033,
                device_longitude=76.1004,
                ip_address="10.23.191.2",
                device_identifier="3c49f915d04485e34caba",
                vpn_used=False,
                operating_system="Android 11.2",
                device_maker="ASUS",
                device_model="Zenphone M2 Pro Max",
                device_year="2018",
                app_version="1.1.0",
            ),
            destination_device_data=DeviceData(
                battery_level=95.0,
                device_latitude=13.0033,
                device_longitude=76.1004,
                ip_address="10.23.191.2",
                device_identifier="3c49f915d04485e34caba",
                vpn_used=False,
                operating_system="Android 11.2",
                device_maker="ASUS",
                device_model="Zenphone M2 Pro Max",
                device_year="2018",
                app_version="1.1.0",
            ),
            tags=[
                Tag(
                    key="customKey",
                    value="customValue",
                )
            ],
        )
        """
        response = self._raw_client.verify(
            type=type,
            transaction_id=transaction_id,
            timestamp=timestamp,
            validate_origin_user_id=validate_origin_user_id,
            validate_destination_user_id=validate_destination_user_id,
            validate_transaction_id=validate_transaction_id,
            trs_only=trs_only,
            origin_user_id=origin_user_id,
            destination_user_id=destination_user_id,
            transaction_state=transaction_state,
            origin_amount_details=origin_amount_details,
            destination_amount_details=destination_amount_details,
            origin_payment_details=origin_payment_details,
            destination_payment_details=destination_payment_details,
            origin_funds_info=origin_funds_info,
            related_transaction_ids=related_transaction_ids,
            product_type=product_type,
            promotion_code_used=promotion_code_used,
            reference=reference,
            origin_device_data=origin_device_data,
            destination_device_data=destination_device_data,
            tags=tags,
            request_options=request_options,
        )
        return response.data

    def get(
        self, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TransactionWithRulesResult:
        """
        ### GET Transactions

        `/transactions` endpoint allows you to operate on the [Transaction entity](/guides/overview/entities#transaction).

        Calling `GET /transactions/{transactionId}` will return the entire transaction payload and rule execution results for the transaction with the corresponding `transactionId`

        Parameters
        ----------
        transaction_id : str
            Unique Transaction Identifier

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransactionWithRulesResult
            OK

        Examples
        --------
        from flagright import Flagright

        client = Flagright(
            api_key="YOUR_API_KEY",
        )
        client.transactions.get(
            transaction_id="transactionId",
        )
        """
        response = self._raw_client.get(transaction_id, request_options=request_options)
        return response.data


class AsyncTransactionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTransactionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTransactionsClient
        """
        return self._raw_client

    async def verify(
        self,
        *,
        type: TransactionType,
        transaction_id: str,
        timestamp: float,
        validate_origin_user_id: typing.Optional[BooleanString] = None,
        validate_destination_user_id: typing.Optional[BooleanString] = None,
        validate_transaction_id: typing.Optional[BooleanString] = None,
        trs_only: typing.Optional[BooleanString] = None,
        origin_user_id: typing.Optional[str] = OMIT,
        destination_user_id: typing.Optional[str] = OMIT,
        transaction_state: typing.Optional[TransactionState] = OMIT,
        origin_amount_details: typing.Optional[TransactionAmountDetails] = OMIT,
        destination_amount_details: typing.Optional[TransactionAmountDetails] = OMIT,
        origin_payment_details: typing.Optional[TransactionOriginPaymentDetails] = OMIT,
        destination_payment_details: typing.Optional[TransactionDestinationPaymentDetails] = OMIT,
        origin_funds_info: typing.Optional[OriginFundsInfo] = OMIT,
        related_transaction_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        product_type: typing.Optional[str] = OMIT,
        promotion_code_used: typing.Optional[bool] = OMIT,
        reference: typing.Optional[str] = OMIT,
        origin_device_data: typing.Optional[DeviceData] = OMIT,
        destination_device_data: typing.Optional[DeviceData] = OMIT,
        tags: typing.Optional[typing.Sequence[Tag]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TransactionsVerifyResponse:
        """
        ## POST Transactions

        `/transactions` endpoint allows you to operate on the [Transaction entity.](/guides/overview/entities#transaction)

        In order to pass the payload of a transaction to Flagright and verify the transaction, you will need to call this endpoint with the transaction payload. Not all fields are mandatory, you will only need to pass in the fields that you have and are relevant for your compliance setup.


        ### Payload

        Here are some of the most used payload fields explained (you can find the full payload [schema below](/api-reference/api-reference/transactions/verify#request) with 1 line descriptions):

        * `type`: Type of transaction (Ex: `WITHDRAWAL`, `DEPOSIT`, `TRANSFER` etc).
        * `transactionId` - Unique Identifier for the transaction.
        * `timestamp` - UNIX timestamp in *milliseconds* of when the transaction took place
        * `transactionState` - The state of the transaction, set to `CREATED` by default. [More details here](/guides/overview/entities#transaction-lifecycle-through-transaction-events)
        * `originUserId` - Unique identifier (if any) of the user who is sending the money. This user must be created within the Flagright system before using the [create a consumer user](/api-reference/api-reference/consumer-users/create) or [create a business user](/api-reference/api-reference/business-users/create) endpoint
        * `destinationUserId` - Unique identifier (if any) of the user who is receiving the money. This user must be created within the Flagright system before using the [create a consumer user](/api-reference/api-reference/consumer-users/create) or [create a business user](/api-reference/api-reference/business-users/create) endpoint
        * `originAmountDetails` - Details of the amount being sent from the origin
        * `destinationAmountDetails` - Details of the amount being received at the destination
        * `originPaymentDetails` - Payment details (if any) used at the origin (ex: `CARD`, `IBAN`, `WALLET` etc). You can click on the dropdown next to the field in the schema below to view all supported payment types.
        * `destinationPaymentDetails` - Payment details (if any) used at the destination (ex: `CARD`, `IBAN`, `WALLET` etc). You can click on the dropdown next to the field in the schema below to view all supported payment types.

        Parameters
        ----------
        type : TransactionType

        transaction_id : str
            Unique transaction identifier

        timestamp : float
            Timestamp of when transaction took place

        validate_origin_user_id : typing.Optional[BooleanString]
            Boolean string whether Flagright should validate if provided originUserId exist. True by default

        validate_destination_user_id : typing.Optional[BooleanString]
            Boolean string whether Flagright should validate if provided destinationUserId exist. True by default

        validate_transaction_id : typing.Optional[BooleanString]
            Boolean string whether Flagright should validate if provided transactionId exist. True by default

        trs_only : typing.Optional[BooleanString]

        origin_user_id : typing.Optional[str]
            UserId for where the transaction originates from

        destination_user_id : typing.Optional[str]
            UserId for transaction's destination. In other words, where the value is being transferred to.

        transaction_state : typing.Optional[TransactionState]

        origin_amount_details : typing.Optional[TransactionAmountDetails]

        destination_amount_details : typing.Optional[TransactionAmountDetails]

        origin_payment_details : typing.Optional[TransactionOriginPaymentDetails]
            Payment details of the origin. It can be a bank account number, wallet ID, card fingerprint etc.

        destination_payment_details : typing.Optional[TransactionDestinationPaymentDetails]

        origin_funds_info : typing.Optional[OriginFundsInfo]

        related_transaction_ids : typing.Optional[typing.Sequence[str]]
            IDs of transactions related to this transaction. Ex: refund, split bills

        product_type : typing.Optional[str]
            Type of produce being used by the consumer (ex wallets, payments etc)

        promotion_code_used : typing.Optional[bool]
            Whether a promotion code was used or not the transaction

        reference : typing.Optional[str]
            Reference field for the transaction indicating the purpose of the transaction etc.

        origin_device_data : typing.Optional[DeviceData]

        destination_device_data : typing.Optional[DeviceData]

        tags : typing.Optional[typing.Sequence[Tag]]
            Additional information that can be added via tags

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransactionsVerifyResponse
            Created

        Examples
        --------
        import asyncio

        from flagright import (
            AsyncFlagright,
            DeviceData,
            Tag,
            TransactionAmountDetails,
            TransactionDestinationPaymentDetails_Card,
            TransactionOriginPaymentDetails_Card,
        )

        client = AsyncFlagright(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.transactions.verify(
                type="DEPOSIT",
                transaction_id="7b80a539eea6e78acbd6d458e5971482",
                timestamp=1641654664000.0,
                origin_user_id="8650a2611d0771cba03310f74bf6",
                destination_user_id="9350a2611e0771cba03310f74bf6",
                origin_amount_details=TransactionAmountDetails(
                    transaction_amount=800.0,
                    transaction_currency="EUR",
                    country="DE",
                ),
                destination_amount_details=TransactionAmountDetails(
                    transaction_amount=68351.34,
                    transaction_currency="INR",
                    country="IN",
                ),
                origin_payment_details=TransactionOriginPaymentDetails_Card(
                    card_fingerprint="20ac00fed8ef913aefb17cfae1097cce",
                    card_issued_country="TR",
                    transaction_reference_field="Deposit",
                    f_3_ds_done=True,
                ),
                destination_payment_details=TransactionDestinationPaymentDetails_Card(
                    card_fingerprint="20ac00fed8ef913aefb17cfae1097cce",
                    card_issued_country="TR",
                    transaction_reference_field="Deposit",
                    f_3_ds_done=True,
                ),
                promotion_code_used=True,
                reference="loan repayment",
                origin_device_data=DeviceData(
                    battery_level=95.0,
                    device_latitude=13.0033,
                    device_longitude=76.1004,
                    ip_address="10.23.191.2",
                    device_identifier="3c49f915d04485e34caba",
                    vpn_used=False,
                    operating_system="Android 11.2",
                    device_maker="ASUS",
                    device_model="Zenphone M2 Pro Max",
                    device_year="2018",
                    app_version="1.1.0",
                ),
                destination_device_data=DeviceData(
                    battery_level=95.0,
                    device_latitude=13.0033,
                    device_longitude=76.1004,
                    ip_address="10.23.191.2",
                    device_identifier="3c49f915d04485e34caba",
                    vpn_used=False,
                    operating_system="Android 11.2",
                    device_maker="ASUS",
                    device_model="Zenphone M2 Pro Max",
                    device_year="2018",
                    app_version="1.1.0",
                ),
                tags=[
                    Tag(
                        key="customKey",
                        value="customValue",
                    )
                ],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.verify(
            type=type,
            transaction_id=transaction_id,
            timestamp=timestamp,
            validate_origin_user_id=validate_origin_user_id,
            validate_destination_user_id=validate_destination_user_id,
            validate_transaction_id=validate_transaction_id,
            trs_only=trs_only,
            origin_user_id=origin_user_id,
            destination_user_id=destination_user_id,
            transaction_state=transaction_state,
            origin_amount_details=origin_amount_details,
            destination_amount_details=destination_amount_details,
            origin_payment_details=origin_payment_details,
            destination_payment_details=destination_payment_details,
            origin_funds_info=origin_funds_info,
            related_transaction_ids=related_transaction_ids,
            product_type=product_type,
            promotion_code_used=promotion_code_used,
            reference=reference,
            origin_device_data=origin_device_data,
            destination_device_data=destination_device_data,
            tags=tags,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TransactionWithRulesResult:
        """
        ### GET Transactions

        `/transactions` endpoint allows you to operate on the [Transaction entity](/guides/overview/entities#transaction).

        Calling `GET /transactions/{transactionId}` will return the entire transaction payload and rule execution results for the transaction with the corresponding `transactionId`

        Parameters
        ----------
        transaction_id : str
            Unique Transaction Identifier

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransactionWithRulesResult
            OK

        Examples
        --------
        import asyncio

        from flagright import AsyncFlagright

        client = AsyncFlagright(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.transactions.get(
                transaction_id="transactionId",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(transaction_id, request_options=request_options)
        return response.data
